@@ -44,6 +44,7 @@ struct st_DBProperties {
   StringX database;
   StringX databaseformat;
   StringX numgroups;
+  StringX numemptygroups;
   StringX numentries;
   StringX numattachments;
   StringX whenlastsaved;
@@ -132,7 +133,7 @@ public:
                const bool bValidate = false, const size_t iMAXCHARS = 0,
                CReport *pRpt = NULL);
   PWSfile::VERSION GetReadFileVersion() const {return m_ReadFileVersion;}
-  bool BackupCurFile(int maxNumIncBackups, int backupSuffix,
+  bool BackupCurFile(unsigned int maxNumIncBackups, int backupSuffix,
                      const stringT &userBackupPrefix,
                      const stringT &userBackupDir, stringT &bu_fname);
 
@@ -142,15 +143,15 @@ public:
                 bool bUpdateSig = true);
   int WriteExportFile(const StringX &filename, OrderedItemList *pOIL,
                       PWScore *pINcore, PWSfile::VERSION version,
-                      CReport *pRpt = NULL);
+                      std::vector<StringX> &vEmptyGroups, CReport *pRpt = NULL);
   int WriteV17File(const StringX &filename)
   {return WriteFile(filename, PWSfile::V17, false);}
   int WriteV2File(const StringX &filename)
   {return WriteFile(filename, PWSfile::V20, false);}
 
   // R/O file status
-  void SetReadOnly(bool state) {m_IsReadOnly = state;}
-  bool IsReadOnly() const {return m_IsReadOnly;};
+  void SetReadOnly(bool state) {m_bIsReadOnly = state;}
+  bool IsReadOnly() const {return m_bIsReadOnly;};
 
   // Check/Change master passphrase
   int CheckPasskey(const StringX &filename, const StringX &passkey);
@@ -184,6 +185,10 @@ public:
                    const int &subgroup_object, const int &subgroup_function,
                    int &numUpdated, CReport *pRpt, bool *pbCancel = NULL);
 
+  // Used for Empty Groups during Merge
+  bool MatchGroupName(const StringX &stValue, const StringX &subgroup_name,
+                      const int &iFunction) const;
+
   // Export databases
   int WritePlaintextFile(const StringX &filename,
                          const CItemData::FieldBits &bsExport,
@@ -196,6 +201,7 @@ public:
                    const CItemData::FieldBits &bsExport,
                    const stringT &subgroup, const int &iObject,
                    const int &iFunction, const TCHAR &delimiter,
+                   const stringT &exportgroup,
                    int &numExported, const OrderedItemList *pOIL = NULL,
                    const bool &bFilterActive = false,
                    CReport *pRpt = NULL);
@@ -210,6 +216,7 @@ public:
                           int &numImported, int &numSkipped,
                           int &numPWHErrors, int &numRenamed, int &numNoPolicy,
                           CReport &rpt, Command *&pcommand);
+
   int ImportXMLFile(const stringT &ImportedPrefix,
                     const stringT &strXMLFileName,
                     const stringT &strXSDFileName,
@@ -219,11 +226,13 @@ public:
                     int &numValidated, int &numImported, int &numSkipped,
                     int &numPWHErrors, int &numRenamed,
                     int &numNoPolicy,  int &numRenamedPolicies,
-                    int &numShortcutsRemoved,
+                    int &numShortcutsRemoved, int &numEmptyGroupsImported,
                     CReport &rpt, Command *&pcommand);
+
   int ImportKeePassV1TXTFile(const StringX &filename,
                              int &numImported, int &numSkipped, int &numRenamed,
                              UINT &uiReasonCode, CReport &rpt, Command *&pcommand);
+
   int ImportKeePassV1CSVFile(const StringX &filename,
                              int &numImported, int &numSkipped, int &numRenamed,
                              UINT &uiReasonCode, CReport &rpt, Command *&pcommand);
@@ -244,8 +253,10 @@ public:
   // Set application data
   void SetApplicationNameAndVersion(const stringT &appName, DWORD dwMajorMinor);
 
-  // Return list of unique groups
-  void GetUniqueGroups(std::vector<stringT> &vUniqueGroups) const;
+  // GetAllGroups - returns an array of all unique group prefix names in m_pwlist
+  // e.g., "A", "A.B", "A.B.C"
+  // "All" includes empty groups!
+  void GetAllGroups(std::vector<stringT> &vAllGroups) const;
   // Construct unique title
   StringX GetUniqueTitle(const StringX &group, const StringX &title,
                          const StringX &user, const int IDS_MESSAGE);
@@ -263,6 +274,9 @@ public:
                         StringX &sxPolicyName, const StringX &sxDateTime,
                         const UINT IDS_MESSAGE);
 
+  bool GetEntriesUsingNamedPasswordPolicy(const StringX sxPolicyName,
+                                          std::vector<st_GroupTitleUser> &ventries);
+
   // Populate setGTU & setUUID from m_pwlist. Returns false & empty set if
   // m_pwlist had one or more entries with same GTU/UUID respectively.
   bool InitialiseGTU(GTUSet &setGTU);
@@ -272,8 +286,6 @@ public:
   // to ensure uniqueness. Returns false if title was modified.
   bool MakeEntryUnique(GTUSet &setGTU, const StringX &group, StringX &title,
                        const StringX &user, const int IDS_MESSAGE);
-  void SetUniqueGTUValidated(bool bState)
-  {m_bUniqueGTUValidated = bState;}
   bool GetUniqueGTUValidated() const
   {return m_bUniqueGTUValidated;}
 
@@ -297,7 +309,6 @@ public:
   void Undo();
   void Redo();
   void ClearCommands();
-  void ResetStateAfterSave();
   bool AnyToUndo() const;
   bool AnyToRedo() const;
 
@@ -333,33 +344,43 @@ public:
   ItemListIter GetUniqueBase(const StringX &grouptitle, 
                              const StringX &titleuser, bool &bMultiple);
 
-  // Use following calls to 'SetChanged' & 'SetDBChanged' sparingly
-  // outside of core
-  void SetChanged(const bool bDBChanged, const bool bDBprefschanged)
-  {m_bDBChanged = bDBChanged; 
-   m_bDBPrefsChanged = bDBprefschanged;
-   NotifyDBModified();}
-  void SetDBChanged(bool bDBChanged, bool bNotify = true)
-  {m_bDBChanged = bDBChanged;
-    if (bNotify) NotifyDBModified();}
-  void SetDBPrefsChanged(bool bDBprefschanged)
-  {m_bDBPrefsChanged = bDBprefschanged;
-   NotifyDBModified();}
+  bool HasDBChanged() const
+  { return m_stDBCS.bDBChanged; }
+  bool HaveDBPrefsChanged() const
+  { return m_stDBCS.bDBPrefsChanged; }
+  bool HaveHeaderPreferencesChanged(const StringX &prefString)
+  { return _tcscmp(prefString.c_str(), m_hdr.m_prefString.c_str()) != 0; }
+  bool HaveEmptyGroupsChanged() const
+  { return m_stDBCS.bEmptyGroupsChanged; }
+  bool HavePasswordPolicyNamesChanged() const
+  { return m_stDBCS.bPolicyNamesChanged; }
+  bool HaveDBFiltersChanged() const
+  { return m_stDBCS.bDBFiltersChanged; }
+
+  // Note GroupDisplay & RUE list not checked for Save Immediately processing
+  // Also, these are changed by user indirect action and therefore changes are NOT
+  // implemented via a Command (do not require Undo/Redo processing)
+  bool HasGroupDisplayChanged() const;
+  bool HasRUEListChanged() const;
+
+  // NOTE - GroupDisplay & RUE list are NOT checked via this call
+  bool HasAnythingChanged() const
+  { return m_stDBCS.HasAnythingChanged(); }
+  
+  // PWScore::Execute uses this to set the changed status
+  void GetChangedStatus(Command *pcmd, st_DBChangeStatus &st_Command);
 
   bool ChangeMode(stringT &locker, int &iErrorCode);
   PWSFileSig& GetCurrentFileSig() {return *m_pFileSig;}
 
-  bool IsChanged() const {return m_bDBChanged;}
-  bool HaveDBPrefsChanged() const {return m_bDBPrefsChanged;}
-  bool HaveHeaderPreferencesChanged(const StringX &prefString)
-  {return _tcscmp(prefString.c_str(), m_hdr.m_prefString.c_str()) != 0;}
-
   // Callback to be notified if the database changes
   void NotifyDBModified();
   void SuspendOnDBNotification()
   {m_bNotifyDB = false;}
   void ResumeOnDBNotification()
   {m_bNotifyDB = true;}
+  bool GetDBNotificationState()
+  {return m_bNotifyDB;}
 
   void GUISetupDisplayInfo(CItemData &ci);
   void GUIRefreshEntry(const CItemData &ci);
@@ -368,26 +389,23 @@ public:
   // Get/Set Display information from/to database
   void SetDisplayStatus(const std::vector<bool> &s);
   const std::vector<bool> &GetDisplayStatus() const;
-  bool WasDisplayStatusChanged() const;
 
   const PWSfileHeader &GetHeader() const {return m_hdr;}
-  void SetHeader(const PWSfileHeader &hdr) { m_hdr = hdr; }
 
   void GetDBProperties(st_DBProperties &st_dbp);
-  void SetHeaderUserFields(st_DBProperties &st_dbp);
+  StringX GetHeaderItem(PWSfile::HeaderType ht);
 
   StringX &GetDBPreferences() {return m_hdr.m_prefString;}
 
-  // Filters
-  PWSFilters m_MapFilters;
+  // Filters, if any
+  const PWSFilters &GetDBFilters()
+  { return m_MapDBFilters; }
 
   // Changed nodes
-  void ClearChangedNodes()
-  {m_vnodes_modified.clear();}
   bool IsNodeModified(StringX &path) const;
 
-  void GetRUEList(UUIDList &RUElist)
-  {RUElist = m_RUEList;}
+  const UUIDList &GetRUEList()
+  {return m_RUEList;}
   void SetRUEList(const UUIDList &RUElist)
   {m_RUEList = RUElist;}
 
@@ -404,29 +422,21 @@ public:
 
   const PSWDPolicyMap &GetPasswordPolicies()
   {return m_MapPSWDPLC;}
-  void SetPasswordPolicies(const PSWDPolicyMap &MapPSWDPLC)
-  {m_MapPSWDPLC = MapPSWDPLC; SetDBChanged(true);}
-
-  void AddPolicy(const StringX &sxPolicyName, const PWPolicy &st_pp,
-                 const bool bAllowReplace = false);
 
   // Empty Groups
-  void SetEmptyGroups(const std::vector<StringX> &vEmptyGroups)
-  {m_vEmptyGroups = vEmptyGroups; SetDBChanged(true);}
-  const std::vector<StringX> &GetEmptyGroups() {return m_vEmptyGroups;}
-  bool IsEmptyGroup(const StringX &sxEmptyGroup);
+  const std::vector<StringX> & GetEmptyGroups() const {return m_vEmptyGroups;}
+  bool IsEmptyGroup(const StringX &sxEmptyGroup) const;
+  size_t GetNumberEmptyGroups() const {return m_vEmptyGroups.size();}
 
-  // Keyboard shortcuts
-  bool AddKBShortcut(const int32 &iKBShortcut, const pws_os::CUUID &uuid);
-  bool DelKBShortcut(const int32 &iKBShortcut, const pws_os::CUUID &uuid);
   const pws_os::CUUID & GetKBShortcut(const int32 &iKBShortcut);
-  const KBShortcutMap &GetAllKBShortcuts() {return m_KBShortcutMap;}
-  void SetAppHotKey(const int32 &iAppHotKey) {m_iAppHotKey = iAppHotKey;}
+  const KBShortcutMap &GetAllKBShortcuts() { return m_KBShortcutMap; }
   int32 GetAppHotKey() const {return m_iAppHotKey;}
+  void SetAppHotKey(const int32 &iAppHotKey) { m_iAppHotKey = iAppHotKey; }
 
   uint32 GetHashIters() const;
   void SetHashIters(uint32 value);
 
+  const CItemAtt &GetAtt(const pws_os::CUUID &attuuid) const {return m_attlist.find(attuuid)->second;}
   CItemAtt &GetAtt(const pws_os::CUUID &attuuid) {return m_attlist[attuuid];}
   void PutAtt(const CItemAtt &att) {m_attlist[att.GetUUID()] = att;}
   void RemoveAtt(const pws_os::CUUID &attuuid);
@@ -463,7 +473,7 @@ private:
                                     SaveTypePWMap *pmapSaveTypePW = NULL);
   virtual void UndoAddDependentEntries(ItemList *pmapDeletedItems,
                                        SaveTypePWMap *pmapSaveTypePW);
-  virtual void DoMoveDependentEntries(const pws_os::CUUID &from_baseuuid, 
+  virtual bool DoMoveDependentEntries(const pws_os::CUUID &from_baseuuid, 
                                       const pws_os::CUUID &to_baseuuid, 
                                       const CItemData::EntryType type);
 
@@ -474,12 +484,46 @@ private:
   virtual int DoRenameGroup(const StringX &sxOldPath, const StringX &sxNewPath);
   virtual void UndoRenameGroup(const StringX &sxOldPath, const StringX &sxNewPath);
 
+  virtual int DoChangeHeader(const StringX &sxNewValue, const PWSfile::HeaderType ht);
+  virtual void UndoChangeHeader(const StringX &sxOldValue, const PWSfile::HeaderType ht);
+
   virtual bool AddEmptyGroup(const StringX &sxEmptyGroup);
   virtual bool RemoveEmptyGroup(const StringX &sxEmptyGroup);
-  virtual void RenameEmptyGroup(const StringX &sxOldPath, const StringX &sxNewPath);
+  virtual bool RenameEmptyGroup(const StringX &sxOldGroup, const StringX &sxNewGroup);
+  virtual bool RenameEmptyGroupPaths(const StringX &sxOldPath, const StringX &sxNewPath);
 
   // End of Command Interface implementations
 
+  //***** Make all calls that change the core private
+  //   This excludes Group Display and RUE List which should not be via 
+  //   Commands as no requirement to Undo/Redo and whose save is UI driven.
+  void SetChangedStatus(); // Used by Execute/Undo/Redo
+  void SetInitialValues(); // Called after successful read/write of a database
+
+  // Update header
+  int SetHeaderItem(const StringX &sxNewValue, PWSfile::HeaderType ht);
+
+  // Update DB preferences - not sure why this is still here (called by DBPrefsCommand::Execute)
+  void SetDBPrefsChanged(bool bDBprefschanged)
+  { m_stDBCS.bDBPrefsChanged = bDBprefschanged; }
+
+  // Set empty groups
+  bool SetEmptyGroups(const std::vector<StringX> &vEmptyGroups);
+
+  // Set/Add Password Policies
+  bool SetPasswordPolicies(const PSWDPolicyMap &MapPSWDPLC);
+  bool AddPolicy(const StringX &sxPolicyName, const PWPolicy &st_pp,
+                const bool bAllowReplace = false);
+
+  // Set DB filters
+  bool SetDBFilters(const PWSFilters &MapDBFilters);
+
+  // Keyboard shortcuts
+  bool AddKBShortcut(const int32 &iKBShortcut, const pws_os::CUUID &uuid);
+  bool DelKBShortcut(const int32 &iKBShortcut, const pws_os::CUUID &uuid);
+
+  //*****
+
   void ProcessReadEntry(CItemData &ci_temp,
                         std::vector<st_GroupTitleUser> &vGTU_INVALID_UUID,
                         std::vector<st_GroupTitleUser> &vGTU_DUPLICATE_UUID,
@@ -491,6 +535,7 @@ private:
   void ResetAllAliasPasswords(const pws_os::CUUID &base_uuid);
   
   StringX GetPassKey() const; // returns cleartext - USE WITH CARE
+
   // Following used by SetPassKey
   void EncryptPassword(const unsigned char *plaintext, size_t len,
                        unsigned char *ciphertext) const;
@@ -508,7 +553,7 @@ private:
   uint32 m_hashIters; // for new or currently open db.
 
   static unsigned char m_session_key[32];
-  static unsigned char m_session_initialized;
+  static bool m_session_initialized;
 
   HANDLE m_lockFileHandle;
   HANDLE m_lockFileHandle2;
@@ -518,13 +563,17 @@ private:
   stringT m_AppNameAndVersion;
   PWSfile::VERSION m_ReadFileVersion;
 
-  bool m_bDBChanged;
-  bool m_bDBPrefsChanged;
-  bool m_IsReadOnly;
+  bool m_bIsReadOnly;
   bool m_bUniqueGTUValidated;
+  bool m_bNotifyDB;
+  bool m_bIsOpen;
+
+  st_DBChangeStatus m_stDBCS;
 
   PWSfileHeader m_hdr;
-  std::vector<bool> m_OrigDisplayStatus;
+  StringX m_InitialDBName, m_InitialDBDesc;
+  StringX m_InitialDBPreferences;
+  std::vector<bool> m_InitialDisplayStatus; // for HasGroupDisplayChanged (stored in header)
 
   // THE password database
   //  Key = entry's uuid; Value = entry's CItemData
@@ -546,23 +595,21 @@ private:
   void SetBase2ShortcutsMmap(ItemMMap &b2smm) {m_base2shortcuts_mmap = b2smm;}
   
   // Changed groups
-  std::vector<StringX> m_vnodes_modified;
+  std::vector<StringX> m_vNodes_Modified;
+  std::vector<StringX> m_saved_vNodes_Modified;
+
   // Following are private in PWScore, public in CommandInterface:
-  virtual const std::vector<StringX> &GetVnodesModified() const
-  {return m_vnodes_modified;}
-  virtual void SetVnodesModified(const std::vector<StringX> &mvm)
-  {m_vnodes_modified = mvm;}
   void AddChangedNodes(StringX path);
 
   // EmptyGroups
   std::vector<StringX> m_vEmptyGroups;
+  std::vector<StringX> m_InitialvEmptyGroups;
 
   UnknownFieldList m_UHFL;
   int m_nRecordsWithUnknownFields;
 
   UUIDList m_RUEList;
-
-  bool m_bNotifyDB;
+  UUIDList m_InitialRUEList;
 
   UIInterFace *m_pUIIF; // pointer to UI interface abtraction
   std::bitset<UIInterFace::NUM_SUPPORTED> m_bsSupportedFunctions;
@@ -596,6 +643,10 @@ private:
 
   stringT GetXMLPWPolicies(const OrderedItemList *pOIL = NULL);
   PSWDPolicyMap m_MapPSWDPLC;
+  PSWDPolicyMap m_InitialMapPSWDPLC;  // Needed for HavePasswordPolicyNamesChanged
+  
+  PWSFilters m_MapDBFilters;  // DB filters only
+  PWSFilters m_InitialMapDBFilters;
 
   KBShortcutMap m_KBShortcutMap;
   int32 m_iAppHotKey;
