@@ -152,7 +152,7 @@ int PWSfileV1V2::Open(const StringX &passkey)
     PWSrand::GetInstance()->GetRandomData( m_ipthing, 8);
     SAFE_FWRITE(m_ipthing, 1, 8, m_fd);
 
-    m_fish = BlowFish::MakeBlowFish(pstr, reinterpret_cast<int &>(passLen),
+    m_fish = BlowFish::MakeBlowFish(pstr, reinterpret_cast<unsigned int &>(passLen),
                                     m_salt, SaltLength);
     if (m_curversion == V20) {
       status = WriteV2Header();
@@ -168,7 +168,7 @@ int PWSfileV1V2::Open(const StringX &passkey)
     fread(m_salt, 1, SaltLength, m_fd);
     fread(m_ipthing, 1, 8, m_fd);
 
-    m_fish = BlowFish::MakeBlowFish(pstr, reinterpret_cast<int &>(passLen),
+    m_fish = BlowFish::MakeBlowFish(pstr, reinterpret_cast<unsigned int &>(passLen),
                                     m_salt, SaltLength);
     if (m_curversion == V20)
       status = ReadV2Header();
