@@ -252,7 +252,7 @@ size_t PWSfileV4::WriteCBC(unsigned char type, const unsigned char *data,
 
   m_hmac.Update(&type, 1);
   m_hmac.Update(buf, sizeof(buf));
-  m_hmac.Update(data, len32);
+  m_hmac.Update(data, length);
 
   return PWSfile::WriteCBC(type, data, length);
 }
@@ -274,7 +274,7 @@ int PWSfileV4::WriteRecord(const CItemAtt &att)
   // Following writes AttIV, AttEK, AttAK, AttContent
   // and AttContentHMAC per format spec.
   // All except the content are generated internally.
-int PWSfileV4::WriteContentFields(unsigned char *content, size_t len)
+size_t PWSfileV4::WriteContentFields(unsigned char *content, size_t len)
 {
   if (len == 0)
     return SUCCESS;
@@ -345,7 +345,7 @@ size_t PWSfileV4::ReadCBC(unsigned char &type, unsigned char* &data,
 
     m_hmac.Update(&type, 1);
     m_hmac.Update(buf, sizeof(buf));
-    m_hmac.Update(data, len32);
+    m_hmac.Update(data, length);
   }
 
   return numRead;
@@ -380,7 +380,7 @@ int PWSfileV4::ReadRecord(CItemData &item)
       RestoreState();
       status = WRONG_RECORD;
     }
-  } else if (unsigned(fpos) == m_effectiveFileLength)
+  } else if (fpos == m_effectiveFileLength)
     status = END_OF_FILE;
   else // fpos >= effectiveFileLength !?
     status = READ_FAIL;
@@ -643,17 +643,17 @@ int PWSfileV4::WriteHeader()
                         m_hdr.m_whatlastsaved);
   if (numWritten <= 0) { status = FAILURE; goto end; }
 
-  if (!m_hdr.m_dbname.empty()) {
-    numWritten = WriteCBC(HDR_DBNAME, m_hdr.m_dbname);
+  if (!m_hdr.m_DB_Name.empty()) {
+    numWritten = WriteCBC(HDR_DBNAME, m_hdr.m_DB_Name);
     if (numWritten <= 0) { status = FAILURE; goto end; }
   }
-  if (!m_hdr.m_dbdesc.empty()) {
-    numWritten = WriteCBC(HDR_DBDESC, m_hdr.m_dbdesc);
+  if (!m_hdr.m_DB_Description.empty()) {
+    numWritten = WriteCBC(HDR_DBDESC, m_hdr.m_DB_Description);
     if (numWritten <= 0) { status = FAILURE; goto end; }
   }
-  if (!m_MapFilters.empty()) {
+  if (!m_MapDBFilters.empty()) {
     coStringXStream oss;  // XML is always char not wchar_t
-    m_MapFilters.WriteFilterXMLFile(oss, m_hdr, _T(""));
+    m_MapDBFilters.WriteFilterXMLFile(oss, m_hdr, _T(""));
     numWritten = WriteCBC(HDR_FILTERS,
                           reinterpret_cast<const unsigned char *>(oss.str().c_str()),
                           oss.str().length());
@@ -665,7 +665,7 @@ int PWSfileV4::WriteHeader()
     if (num > 255)
       num = 255;  // Only save up to max as defined by FormatV3.
 
-    int buflen = (num * sizeof(uuid_array_t)) + 1;
+    size_t buflen = (num * sizeof(uuid_array_t)) + 1;
     unsigned char *buf = new unsigned char[buflen];
     buf[0] = (unsigned char)num;
     unsigned char *buf_ptr = buf + 1;
@@ -709,7 +709,7 @@ int PWSfileV4::WriteHeader()
 
     // Allocate buffer in calculated size
     unsigned char *buf = new unsigned char[totlen];
-    memset(buf, 0, totlen); // in case we trucate some names, don't leak info.
+    memset(buf, 0, totlen); // in case we truncate some names, don't leak info.
 
     // fill buffer
     buf[0] = (unsigned char)numPols;
@@ -887,7 +887,7 @@ int PWSfileV4::ParseKeyBlocks(const StringX &passkey)
 
   do {
     status = ReadKeyBlock();
-    // status is either SUCESS or END_OF_FILE
+    // status is either SUCCESS or END_OF_FILE
     if (status == END_OF_FILE) {
       return status;
     }
@@ -962,7 +962,7 @@ bool PWSfileV4::CKeyBlocks::RemoveKeyBlock(const StringX &passkey)
     return false;
 
   KeyBlockFinder find_kb(passkey);
-  const unsigned old_size = m_kbs.size();
+  const unsigned long old_size = m_kbs.size();
   m_kbs.erase(remove_if(m_kbs.begin(), m_kbs.end(), find_kb),
                m_kbs.end());
 
@@ -1086,13 +1086,13 @@ int PWSfileV4::ReadHeader()
     case HDR_DBNAME:
       if (utf8 != NULL) utf8[utf8Len] = '\0';
       utf8status = m_utf8conv.FromUTF8(utf8, utf8Len, text);
-      m_hdr.m_dbname = text;
+      m_hdr.m_DB_Name = text;
       break;
 
     case HDR_DBDESC:
       if (utf8 != NULL) utf8[utf8Len] = '\0';
       utf8status = m_utf8conv.FromUTF8(utf8, utf8Len, text);
-      m_hdr.m_dbdesc = text;
+      m_hdr.m_DB_Description = text;
       break;
 
 #if !defined(USE_XML_LIBRARY) || (!defined(_WIN32) && USE_XML_LIBRARY == MSXML)
@@ -1123,7 +1123,7 @@ int PWSfileV4::ReadHeader()
           m_UHFL.push_back(unkhfe);
           break;
         }
-        int rc = m_MapFilters.ImportFilterXMLFile(FPOOL_DATABASE, text.c_str(), _T(""),
+        int rc = m_MapDBFilters.ImportFilterXMLFile(FPOOL_DATABASE, text.c_str(), _T(""),
                                                   XSDFilename.c_str(),
                                                   strErrors, m_pAsker);
         if (rc != PWScore::SUCCESS) {
