@@ -92,6 +92,7 @@ bool XMLFileHandlers::ProcessStartElement(const int icurrent_element)
       m_numNoPolicies = 0;
       m_numRenamedPolicies = 0;
       m_numShortcutsRemoved = 0;
+      m_numEmptyGroupsImported = 0;
       m_bEntryBeingProcessed = false;
       break;
     case XLE_ENTRY:
@@ -195,7 +196,7 @@ void XMLFileHandlers::ProcessEndElement(const int icurrent_element)
       bpref = PWSprefs::ShowPasswordInTree;
       break;
     case XLE_PREF_SORTASCENDING:
-      bpref = PWSprefs::SortAscending;
+      // Obsolete in 3.40 - keep but do nothing
       break;
     case XLE_PREF_USEDEFAULTUSER:
       bpref = PWSprefs::UseDefaultUser;
@@ -352,8 +353,11 @@ void XMLFileHandlers::ProcessEndElement(const int icurrent_element)
       break;
     case XLE_EGNAME:
       if (!m_sxElemContent.empty() &&
-          find(m_vEmptyGroups.begin(), m_vEmptyGroups.end(), m_sxElemContent) == m_vEmptyGroups.end())
+          find(m_vEmptyGroups.begin(), m_vEmptyGroups.end(), m_sxElemContent) 
+                  == m_vEmptyGroups.end()) {
         m_vEmptyGroups.push_back(m_sxElemContent);
+        m_numEmptyGroupsImported++;
+      }
       break;
 
     // MUST be in the same order as enum beginning STR_GROUP...
@@ -602,6 +606,12 @@ void XMLFileHandlers::AddXMLEntries()
 
   // Then add any Empty Groups imported that are not already in the database
   if (!m_vEmptyGroups.empty()) {
+    if (!m_ImportedPrefix.empty()) {
+      const StringX sxNewPath = StringX(m_ImportedPrefix.c_str()) + StringX(L".");
+      for (size_t i = 0; i < m_vEmptyGroups.size(); i++) {
+        m_vEmptyGroups[i] = sxNewPath + m_vEmptyGroups[i];
+      }
+    }
     Command *pcmd = DBEmptyGroupsCommand::Create(m_pXMLcore, m_vEmptyGroups,
                            DBEmptyGroupsCommand::EG_ADDALL);
     m_pmulticmds->Add(pcmd);
