@@ -263,7 +263,7 @@ void CManagePasswordPolicies::CreateControls()
     FindWindow(wxID_CANCEL)->Show(false);
 
     FindWindow(wxID_OK)->SetLabel(_("Close"));
-    FindWindow(wxID_EDIT)->SetLabel(_("View"));
+    FindWindow(ID_EDIT_PP)->SetLabel(_("View"));
   }
 
   // We have 2 grids, but we show only one at a time,
@@ -401,7 +401,7 @@ void CManagePasswordPolicies::UpdateNames()
     m_PolicyNames->SetCellValue(nPos, 0, iter->first.c_str());
     wxString useCount;
     if (iter->second.usecount != 0)
-      useCount.Printf(wxT("%d"),iter->second.usecount);
+      useCount << iter->second.usecount;
     else
       useCount = _("Not used");
     m_PolicyNames->SetCellValue(nPos, 1, useCount);
@@ -533,7 +533,7 @@ void CManagePasswordPolicies::OnNewClick( wxCommandEvent& )
 {
   CPasswordPolicy ppdlg(this, m_core, m_MapPSWDPLC);
   PWPolicy st_pp = m_st_default_pp;
-  ppdlg.SetPolicyData(wxT(""), st_pp);
+  ppdlg.SetPolicyData(wxEmptyString, st_pp);
   if (ppdlg.ShowModal() == wxID_OK) {
     wxString policyname;
 
@@ -636,8 +636,8 @@ void CManagePasswordPolicies::OnRedoClick( wxCommandEvent& event )
 void CManagePasswordPolicies::OnOkClick( wxCommandEvent& )
 {
   /*
-   * User may have changed default poliicy, named policy, none or both.
-   * If anything has changed, we trat the change as atomic, creating a multicommand
+   * User may have changed default policy, named policy, none or both.
+   * If anything has changed, we treat the change as atomic, creating a multicommand
    * s.t. Undo/Redo will work as expected.
    */
   PWPolicy olddefpol(PWSprefs::GetInstance()->GetDefaultPolicy());
