@@ -27,7 +27,7 @@
 #include <vector>
 
 #include "PWStree.h"
-#include "passwordsafeframe.h" // for DispatchDblClickAction()
+#include "passwordsafeframe.h"
 #include "core/PWSprefs.h"
 #include "../../core/Command.h"
 
@@ -89,6 +89,7 @@ enum {ID_TREECTRL_1 = ID_TREECTRL + 1 };
 BEGIN_EVENT_TABLE( PWSTreeCtrl, wxTreeCtrl )
 
 ////@begin PWSTreeCtrl event table entries
+  EVT_TREE_SEL_CHANGED( ID_TREECTRL, PWSTreeCtrl::OnTreectrlSelChanged )
   EVT_TREE_ITEM_ACTIVATED( ID_TREECTRL, PWSTreeCtrl::OnTreectrlItemActivated )
   EVT_TREE_ITEM_MENU( ID_TREECTRL, PWSTreeCtrl::OnContextMenu )
   EVT_CHAR( PWSTreeCtrl::OnChar )
@@ -235,7 +236,7 @@ static StringX GetPathElem(StringX &sxPath)
   size_t len=sxPath.length();
   if (dotPos == StringX::npos){
     sxElement = sxPath;
-    sxPath = wxT("");
+    sxPath = wxEmptyString;
   } else {
     while ((dotPos < len) && (sxPath[dotPos] == GROUP_SEP)) {// look for consecutive dots
       dotPos++;
@@ -246,7 +247,7 @@ static StringX GetPathElem(StringX &sxPath)
     }
     else { // trailing dots
       sxElement = sxPath;
-      sxPath = wxT("");
+      sxPath = wxEmptyString;
     }
   }
   return sxElement;
@@ -386,7 +387,6 @@ void PWSTreeCtrl::UpdateItemField(const CItemData &item, CItemData::FieldType ft
   else if (ft == CItemData::TITLE || ft == CItemData::START ||
        (ft == CItemData::USER && prefs->GetPref(PWSprefs::ShowUsernameInTree)) ||
        (ft == CItemData::PASSWORD && prefs->GetPref(PWSprefs::ShowPasswordInTree))) {
-    wxRect rc;
     wxTreeItemId ti = Find(item);
     if (ti.IsOk()) {
       SetItemText(ti, ItemDisplayString(item));
@@ -424,7 +424,7 @@ CItemData *PWSTreeCtrl::GetItem(const wxTreeItemId &id) const
 
 }
 
-//overriden from base for case-insensitive sort
+//overridden from base for case-insensitive sort
 int PWSTreeCtrl::OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId& item2)
 {
   const bool groupsFirst = PWSprefs::GetInstance()->GetPref(PWSprefs::ExplorerTypeTree),
@@ -650,10 +650,18 @@ void PWSTreeCtrl::OnRenameGroup(wxCommandEvent& /* evt */)
 
 void PWSTreeCtrl::OnEndLabelEdit( wxTreeEvent& evt )
 {
+  const wxString &label =evt.GetLabel();
+
+  if (label.empty()) {
+    // empty entry or group names are a non-no...
+    evt.Veto();
+    return;
+  }
+
   switch (evt.GetId()) {
     case ID_TREECTRL:
     {
-      if (evt.GetLabel().Find(wxT('.')) == wxNOT_FOUND) {
+      if (label.Find(wxT('.')) == wxNOT_FOUND) {
       // Not safe to modify the tree ctrl in any way.  Wait for the stack to unwind.
       wxTreeEvent newEvt(evt);
       newEvt.SetId(ID_TREECTRL_1);
@@ -748,7 +756,7 @@ void PWSTreeCtrl::FinishRenamingGroup(wxTreeEvent& evt, wxTreeItemId groupItem,
     if (*itr == sxOldPath || itr->find(sxOldPathWithDot) == 0) {
       StringX sxOld = *itr;
       StringX sxNew = sxNewPath + itr->substr(sxOldPath.size());
-      pmcmd->Add(DBEmptyGroupsCommand::Create(&m_core, sxOld, sxNew));
+      pmcmd->Add(DBEmptyGroupsCommand::Create(&m_core, sxOld, sxNew, DBEmptyGroupsCommand::EG_RENAME));
     }
   }
 
@@ -761,3 +769,37 @@ void PWSTreeCtrl::FinishRenamingGroup(wxTreeEvent& evt, wxTreeItemId groupItem,
     wxTreeCtrl::SelectItem(newItem);
 }
 
+
+/*!
+ * wxEVT_COMMAND_TREE_SEL_CHANGED event handler for ID_TREECTRL
+ */
+
+void PWSTreeCtrl::OnTreectrlSelChanged( wxTreeEvent& evt )
+{
+  CItemData *pci = GetItem(evt.GetItem());
+
+  dynamic_cast<PasswordSafeFrame *>(GetParent())->UpdateSelChanged(pci);
+}
+
+static void ColourChildren(PWSTreeCtrl *tree, wxTreeItemId parent, const wxColour &colour)
+{
+  wxTreeItemIdValue cookie;
+  wxTreeItemId child = tree->GetFirstChild(parent, cookie);
+
+  while (child) {
+    tree->SetItemTextColour(child, colour);
+    child = tree->GetNextChild(parent, cookie);
+    if (child && tree->ItemHasChildren(child))
+      ColourChildren(tree, child, colour);
+  }
+}
+
+void PWSTreeCtrl::SetFilterState(bool state)
+{
+  const wxColour *colour = state ? wxRED : wxBLACK;
+  // iterate over all items, no way to do this en-mass
+  wxTreeItemId root = GetRootItem();
+  if (root)
+    ColourChildren(this, root, *colour);
+}
+
