@@ -137,7 +137,8 @@ bool AddEditPropSheet::Create( wxWindow* parent, wxWindowID id, const wxString&
   SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY|wxWS_EX_BLOCK_EVENTS);
   wxPropertySheetDialog::Create( parent, id, caption, pos, size, style );
 
-  CreateButtons(wxOK|wxCANCEL|wxHELP);
+  int flags = (m_type == VIEW) ? (wxCLOSE|wxHELP) : (wxOK|wxCANCEL|wxHELP);
+  CreateButtons(flags);
   CreateControls();
   LayoutDialog();
   Centre();
@@ -866,7 +867,7 @@ void AddEditPropSheet::ItemFieldsToPropSheet()
   std::vector<stringT>::iterator sviter;
 
   // Populate the group combo box
-  m_core.GetUniqueGroups(svec);
+  m_core.GetAllGroups(svec);
   for (sviter = svec.begin(); sviter != svec.end(); sviter++)
     m_groupCtrl->Append(sviter->c_str());
 
